@@ -64,7 +64,6 @@ extern "C" {
 #endif
 
 #if (MYSQL_VERSION_ID >= 50607)
-// TODO
 #  define MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER 1
 #  define MRN_HANDLER_HAVE_HA_PREPARE_INPLACE_ALTER_TABLE 1
 #  define MRN_HANDLER_HAVE_HA_INPLACE_ALTER_TABLE 1
@@ -114,6 +113,10 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_PREPARE_FOR_FORCED_CLOSE
 #endif
 
+#if MYSQL_VERSION_ID >= 50607
+#  define MRN_HAVE_HA_EXTRA_EXPORT
+#endif
+
 #if MYSQL_VERSION_ID >= 50604
 #  define MRN_TIMESTAMP_USE_TIMEVAL
 #elif defined(MRN_MARIADB_P)
@@ -136,6 +139,14 @@ extern "C" {
 #  define MRN_HANDLER_AUTO_REPAIR_HAVE_ERROR
 #endif
 
+#if MYSQL_VERSION_ID >= 50604
+#  define MRN_JOIN_TAB_HAVE_CONDITION
+#endif
+
+#if MYSQL_VERSION_ID >= 50607
+#  define MRN_ITEM_HAVE_ITEM_NAME
+#endif
+
 #if MYSQL_VERSION_ID < 50600
 #  define MRN_RBR_UPDATE_NEED_ALL_COLUMNS
 #endif
@@ -149,10 +160,14 @@ extern "C" {
 #  define MRN_HAVE_HA_REBIND_PSI
 #endif
 
-#if MYSQL_VERSION_ID >= 50500
-#  define MRN_GET_ERROR_MESSAGE current_thd->stmt_da->message()
+#if MYSQL_VERSION_ID >= 50607
+#  define MRN_GET_ERROR_MESSAGE current_thd->get_stmt_da()->message()
 #else
-#  define MRN_GET_ERROR_MESSAGE current_thd->main_da.message()
+#  if MYSQL_VERSION_ID >= 50500
+#    define MRN_GET_ERROR_MESSAGE current_thd->stmt_da->message()
+#  else
+#    define MRN_GET_ERROR_MESSAGE current_thd->main_da.message()
+#  endif
 #endif
 
 #if defined(_WIN32) || defined(_WIN64)
@@ -167,6 +182,9 @@ class ha_mroonga;
 struct st_mrn_ft_info
 {
   struct _ft_vft *please;
+#ifdef HA_CAN_FULLTEXT_EXT
+  struct _ft_vft_ext *could_you;
+#endif
   grn_ctx *ctx;
   grn_obj *table;
   grn_obj *result;
@@ -211,6 +229,19 @@ private:
   mutable handler     *wrap_handler_for_create;
 #ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
   handler_add_index *hnd_add_index;
+#endif
+#ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
+  Alter_inplace_info::HA_ALTER_FLAGS alter_handler_flags;
+  KEY         *alter_key_info_buffer;
+  uint        alter_key_count;
+  uint        alter_index_drop_count;
+  KEY         *alter_index_drop_buffer;
+  uint        alter_index_add_count;
+  uint        *alter_index_add_buffer;
+  TABLE       *wrap_altered_table;
+  KEY         *wrap_altered_table_key_info;
+  TABLE_SHARE *wrap_altered_table_share;
+  KEY         *wrap_altered_table_share_key_info;
 #endif
   int mrn_lock_type;
 
@@ -405,16 +436,22 @@ public:
   bool is_fatal_error(int error_num, uint flags);
   bool check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                   uint table_changes);
+#ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
+  enum_alter_inplace_result
+  check_if_supported_inplace_alter(TABLE *altered_table,
+                                   Alter_inplace_info *ha_alter_info);
+#else
   uint alter_table_flags(uint flags);
-#ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
+#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
   int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
                 handler_add_index **add);
   int final_add_index(handler_add_index *add, bool commit);
-#else
+#  else
   int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-#endif
+#  endif
   int prepare_drop_index(TABLE *table_arg, uint *key_num, uint num_of_keys);
   int final_drop_index(TABLE *table_arg);
+#endif
   int update_auto_increment();
   void set_next_insert_id(ulonglong id);
   void get_auto_increment(ulonglong offset, ulonglong increment, ulonglong nb_desired_values,
@@ -476,6 +513,16 @@ protected:
                                      uint key_length,
                                      qc_engine_callback *engine_callback,
                                      ulonglong *engine_data);
+#ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
+  bool prepare_inplace_alter_table(TABLE *altered_table,
+                                   Alter_inplace_info *ha_alter_info);
+  bool inplace_alter_table(TABLE *altered_table,
+                           Alter_inplace_info *ha_alter_info);
+  bool commit_inplace_alter_table(TABLE *altered_table,
+                                  Alter_inplace_info *ha_alter_info,
+                                  bool commit);
+  void notify_table_changed();
+#endif
 
 private:
   void mkdir_p(const char *directory);
@@ -514,7 +561,8 @@ private:
 
   long long int get_grn_time_from_timestamp_field(Field_timestamp *field);
 
-  int generic_store_bulk_string(Field *field, grn_obj *buf);
+  int generic_store_bulk_fixed_size_string(Field *field, grn_obj *buf);
+  int generic_store_bulk_variable_size_string(Field *field, grn_obj *buf);
   int generic_store_bulk_integer(Field *field, grn_obj *buf);
   int generic_store_bulk_unsigned_integer(Field *field, grn_obj *buf);
   int generic_store_bulk_float(Field *field, grn_obj *buf);
@@ -572,12 +620,26 @@ private:
                                             grn_id record_id);
   void storage_store_fields_by_index(uchar *buf);
 
+  int storage_encode_key_fixed_size_string(Field *field, const uchar *key,
+                                           uchar *buf, uint *size);
+  int storage_encode_key_variable_size_string(Field *field, const uchar *key,
+                                              uchar *buf, uint *size);
+  int storage_encode_key_timestamp(Field *field, const uchar *key,
+                                   uchar *buf, uint *size);
   int storage_encode_key_time(Field *field, const uchar *key,
                               uchar *buf, uint *size);
   int storage_encode_key_year(Field *field, const uchar *key,
                               uchar *buf, uint *size);
   int storage_encode_key_datetime(Field *field, const uchar *key,
                                   uchar *buf, uint *size);
+#ifdef MRN_HAVE_MYSQL_TYPE_TIMESTAMP2
+  int storage_encode_key_timestamp2(Field *field, const uchar *key,
+                                    uchar *buf, uint *size);
+#endif
+#ifdef MRN_HAVE_MYSQL_TYPE_DATETIME2
+  int storage_encode_key_datetime2(Field *field, const uchar *key,
+                                   uchar *buf, uint *size);
+#endif
 #ifdef MRN_HAVE_MYSQL_TYPE_TIME2
   int storage_encode_key_time2(Field *field, const uchar *key,
                                uchar *buf, uint *size);
@@ -906,31 +968,56 @@ private:
                                           uint table_changes);
   bool storage_check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                           uint table_changes);
-  uint wrapper_alter_table_flags(uint flags);
-  uint storage_alter_table_flags(uint flags);
-#ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
-  int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
-                        handler_add_index **add);
-  int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
-                        handler_add_index **add);
-#else
-  int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-  int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-#endif
   int storage_add_index_multiple_columns(KEY *key_info, uint num_of_keys,
                                          grn_obj **index_tables,
                                          grn_obj **index_columns,
                                          bool skip_unique_key);
-#ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
+#ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
+  enum_alter_inplace_result
+  wrapper_check_if_supported_inplace_alter(TABLE *altered_table,
+                                           Alter_inplace_info *ha_alter_info);
+  enum_alter_inplace_result
+  storage_check_if_supported_inplace_alter(TABLE *altered_table,
+                                           Alter_inplace_info *ha_alter_info);
+  bool wrapper_prepare_inplace_alter_table(TABLE *altered_table,
+                                           Alter_inplace_info *ha_alter_info);
+  bool storage_prepare_inplace_alter_table(TABLE *altered_table,
+                                           Alter_inplace_info *ha_alter_info);
+  bool wrapper_inplace_alter_table(TABLE *altered_table,
+                                   Alter_inplace_info *ha_alter_info);
+  bool storage_inplace_alter_table(TABLE *altered_table,
+                                   Alter_inplace_info *ha_alter_info);
+  bool wrapper_commit_inplace_alter_table(TABLE *altered_table,
+                                          Alter_inplace_info *ha_alter_info,
+                                          bool commit);
+  bool storage_commit_inplace_alter_table(TABLE *altered_table,
+                                          Alter_inplace_info *ha_alter_info,
+                                          bool commit);
+  void wrapper_notify_table_changed();
+  void storage_notify_table_changed();
+#else
+  uint wrapper_alter_table_flags(uint flags);
+  uint storage_alter_table_flags(uint flags);
+#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
+  int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
+                        handler_add_index **add);
+  int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
+                        handler_add_index **add);
+#  else
+  int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
+  int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
+#  endif
+#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
   int wrapper_final_add_index(handler_add_index *add, bool commit);
   int storage_final_add_index(handler_add_index *add, bool commit);
-#endif
+#  endif
   int wrapper_prepare_drop_index(TABLE *table_arg, uint *key_num,
                                  uint num_of_keys);
   int storage_prepare_drop_index(TABLE *table_arg, uint *key_num,
                                  uint num_of_keys);
   int wrapper_final_drop_index(TABLE *table_arg);
   int storage_final_drop_index(TABLE *table_arg);
+#endif
   int wrapper_update_auto_increment();
   int storage_update_auto_increment();
   void wrapper_set_next_insert_id(ulonglong id);
@@ -1006,11 +1093,6 @@ private:
                                              qc_engine_callback
                                              *engine_callback,
                                              ulonglong *engine_data);
-  COND *get_select_cond();
-  Item_func_match *get_tgt_ft_item(Item_func_match *current);
-  bool check_ft_in_where_item_type(Item *item, Item *ft_item);
-  bool check_ft_in_where_item_cond(Item_cond *item_cond, Item *ft_item);
-  bool check_ft_in_where_item_func(Item_func *item_func, Item *ft_item);
 };
 
 #ifdef __cplusplus
