@@ -35,32 +35,14 @@
 #  define MRN_HAVE_MYSQL_TYPE_TIME2
 #endif
 
-#ifndef HA_INPLACE_ADD_INDEX_NO_READ_WRITE
-#  define HA_INPLACE_ADD_INDEX_NO_READ_WRITE         HA_ONLINE_ADD_INDEX_NO_WRITES
-#  define HA_INPLACE_DROP_INDEX_NO_READ_WRITE        HA_ONLINE_DROP_INDEX_NO_WRITES
-#  define HA_INPLACE_ADD_UNIQUE_INDEX_NO_READ_WRITE  HA_ONLINE_ADD_UNIQUE_INDEX_NO_WRITES
-#  define HA_INPLACE_DROP_UNIQUE_INDEX_NO_READ_WRITE HA_ONLINE_DROP_UNIQUE_INDEX_NO_WRITES
-#  define HA_INPLACE_ADD_PK_INDEX_NO_READ_WRITE      HA_ONLINE_ADD_PK_INDEX_NO_WRITES
-#  define HA_INPLACE_DROP_PK_INDEX_NO_READ_WRITE     HA_ONLINE_DROP_PK_INDEX_NO_WRITES
-
-#  define HA_INPLACE_ADD_INDEX_NO_WRITE              HA_ONLINE_ADD_INDEX
-#  define HA_INPLACE_DROP_INDEX_NO_WRITE             HA_ONLINE_DROP_INDEX
-#  define HA_INPLACE_ADD_UNIQUE_INDEX_NO_WRITE       HA_ONLINE_ADD_UNIQUE_INDEX
-#  define HA_INPLACE_DROP_UNIQUE_INDEX_NO_WRITE      HA_ONLINE_DROP_UNIQUE_INDEX
-#  define HA_INPLACE_ADD_PK_INDEX_NO_WRITE           HA_ONLINE_ADD_PK_INDEX
-#  define HA_INPLACE_DROP_PK_INDEX_NO_WRITE          HA_ONLINE_DROP_PK_INDEX
-#endif
-
-#if MYSQL_VERSION_ID < 50600
-  typedef Item COND;
-#endif
-
 #if MYSQL_VERSION_ID < 50603
   typedef MYSQL_ERROR Sql_condition;
 #endif
 
-#if (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 50302)
-  typedef COST_VECT Cost_estimate;
+#if defined(MRN_MARIADB_P)
+#  if MYSQL_VERSION_ID >= 50302 && MYSQL_VERSION_ID < 100000
+     typedef COST_VECT Cost_estimate;
+#  endif
 #endif
 
 #if MYSQL_VERSION_ID >= 50516
@@ -76,4 +58,14 @@
   typedef char *range_id_t;
 #endif
 
+#if MYSQL_VERSION_ID >= 50609
+#  define MRN_KEY_HAS_USER_DEFINED_KEYPARTS
+#endif
+
+#ifdef MRN_KEY_HAS_USER_DEFINED_KEYPARTS
+#  define KEY_N_KEY_PARTS(key) (key)->user_defined_key_parts
+#else
+#  define KEY_N_KEY_PARTS(key) (key)->key_parts
+#endif
+
 #endif /* _mrn_mysql_compat_h */
