@@ -395,6 +395,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
       if (*tmp_ptr == '\0')
         continue;
 
+      DBUG_PRINT("info", ("mroonga title_str=%s", tmp_ptr));
       title_length = 0;
       start_ptr = tmp_ptr;
       while (*start_ptr != ' ' && *start_ptr != '\'' &&
@@ -405,6 +406,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
         title_length++;
         start_ptr++;
       }
+      DBUG_PRINT("info", ("mroonga title_length=%u", title_length));
 
       switch (title_length)
       {
@@ -479,10 +481,10 @@ error_alloc_param_string:
   DBUG_RETURN(error);
 }
 
-bool mrn_is_geo_key(KEY *key_info)
+bool mrn_is_geo_key(const KEY *key_info)
 {
   return key_info->algorithm == HA_KEY_ALG_UNDEF &&
-    key_info->key_parts == 1 &&
+    KEY_N_KEY_PARTS(key_info) == 1 &&
     key_info->key_part[0].field->type() == MYSQL_TYPE_GEOMETRY;
 }
 
@@ -863,7 +865,7 @@ void mrn_set_bitmap_by_key(MY_BITMAP *map, KEY *key_info)
 {
   uint i;
   DBUG_ENTER("mrn_set_bitmap_by_key");
-  for (i = 0; i < key_info->key_parts; i++)
+  for (i = 0; i < KEY_N_KEY_PARTS(key_info); i++)
   {
     Field *field = key_info->key_part[i].field;
     bitmap_set_bit(map, field->field_index);
